/**
 * 
 */
package org.springblade.cube.controller;

import java.io.File;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.UUID;
import java.util.stream.Stream;

import org.apache.commons.io.FileUtils;
import org.apache.commons.io.FilenameUtils;
import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.secure.utils.SecureUtil;
import org.springblade.core.tool.api.R;
import org.springblade.cube.enums.DbCreateTableEnum;
import org.springblade.cube.enums.ViewColumnTypeEnum;
import org.springblade.cube.exception.ExcelReadLimitException;
import org.springblade.cube.model.CubeTableFile;
import org.springblade.cube.model.FileStructure;
import org.springblade.cube.service.CubeTableFileService;
import org.springblade.cube.service.DynamicTableService;
import org.springblade.cube.utils.CubeConstants;
import org.springblade.cube.utils.CubeExcelAnalysisEventListener;
import org.springblade.cube.utils.CubeUtils;
import org.springblade.report.entity.ReportDatasource;
import org.springblade.report.enums.DsTypeEnum;
import org.springblade.report.service.IReportDatasourceService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import com.alibaba.excel.EasyExcel;
import com.alibaba.fastjson2.JSON;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;

/**
 * @author wookvn
 *
 */
@Slf4j
@RestController
@RequestMapping("/cube/file")
@Api(value = "数据集文件接口", tags = "数据集_文件上传接口")
public class CubeFileController extends BladeController {

	@Autowired
	private IReportDatasourceService reportDatasourceService;
	
	@Autowired
	private DynamicTableService dynamicTableService;
	
	@Autowired
	private CubeTableFileService cubeTableFileService;
	
	/**
	 * 文件结构
	 * @param file
	 * @return
	 * @throws Exception 
	 */
	@ApiOperation(value = "文件结构接口")
	@PostMapping("/structure")
	public R<List<FileStructure>> fileStructure(MultipartFile file) throws Exception{
		//1.读取并解析excel文件
		String extension = FilenameUtils.getExtension(file.getOriginalFilename());
		if(!Stream.of(CubeConstants.SUPPORTS_FILE).anyMatch(f -> f.equalsIgnoreCase(extension))) {
			return R.fail(String.format("文件只支持%s格式", String.join(",", CubeConstants.SUPPORTS_FILE)));
		}
		//数据记录集
		List<Map<Integer, String>> dataList = new ArrayList<Map<Integer,String>>();
		CubeExcelAnalysisEventListener analysisEventListener = new CubeExcelAnalysisEventListener(10, false, dataList::addAll);
		try {
			//读取数据
			EasyExcel.read(file.getInputStream(), analysisEventListener).sheet().doRead();
		} catch (Exception e) {
			if(e.getCause() instanceof ExcelReadLimitException || e instanceof ExcelReadLimitException) {
			}else {
				log.error(e.getMessage(), e);
			}
		}
		//表头数据
		Map<Integer, String> headDataMap = analysisEventListener.getHeadMap();
		List<FileStructure> fileStructList = new ArrayList<FileStructure>();
		//表头和数据集不为空
		if(headDataMap != null && !headDataMap.isEmpty() && !dataList.isEmpty()) {
			headDataMap.forEach((k, v) -> {
				FileStructure fileStructure = new FileStructure();
				fileStructure.setColumnName(CubeConstants.COLUMN_NAME_PREFIX + k);
				fileStructure.setColumnType(CubeUtils.resolveDataType(k, dataList));
				fileStructure.setComment(v);
				fileStructure.setColumnIndex(k);
				fileStructList.add(fileStructure);
			});
		}
		return R.data(fileStructList);
	}
	
	@Value("${blade.file.uploadCube}")
	private String cubeDir;
	
	/**
	 * 文件上传创建表
	 * @return
	 */
	@ApiOperation(value = "文件上传创建表")
	@PostMapping("/uploadToDs")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "file", value = "文件", required = true),
		@ApiImplicitParam(name = "tableName", value = "表名", dataType = "string", required = true),
		@ApiImplicitParam(name = "tableComment", value = "表注释", required = true),
		@ApiImplicitParam(name = "dsId", value = "数据源ID", required = true),
		@ApiImplicitParam(name = "tableColumnList", value = "表字段数组json", required = true)
	})
	public R<String> uploadToDs(MultipartFile file, String tableName, String tableComment, Integer dsId, String tableColumnList){
		String extension = FilenameUtils.getExtension(file.getOriginalFilename());
		if(!Stream.of(CubeConstants.SUPPORTS_FILE).anyMatch(f -> f.equalsIgnoreCase(extension))) {
			return R.fail(String.format("文件只支持%s格式", String.join(",", CubeConstants.SUPPORTS_FILE)));
		}
		try {
			//1.查询数据源
			ReportDatasource reportDatasource = reportDatasourceService.getById(dsId);
			if(reportDatasource == null) {
				return R.fail("数据源不存在");
			}
			DsTypeEnum dsTypeEnum = DsTypeEnum.resolve(reportDatasource.getDsType());
			
			//表名需要特殊处理[ORACLE数据库,表名变大写]
			if(dsTypeEnum == DsTypeEnum.oracle) {
				tableName = tableName.toUpperCase();
			}
			//保存文件
			String tempFileName = UUID.randomUUID().toString() + "." + extension;
			File cubeFile = new File(cubeDir + tempFileName);
			//保存文件
			FileUtils.copyInputStreamToFile(file.getInputStream(), cubeFile);
			//初始化文件对象
			CubeTableFile cubeTableFile = new CubeTableFile();
			cubeTableFile.setCreateTime(new Date());
			cubeTableFile.setCreateBy(SecureUtil.getNickName());
			cubeTableFile.setCreateUserId(SecureUtil.getUserId());
			cubeTableFile.setFileName(file.getOriginalFilename());
			cubeTableFile.setDatasourceId(dsId);
			cubeTableFile.setTableComment(tableComment);
			cubeTableFile.setTableName(tableName);
			cubeTableFile.setFileUri("uploads/cube/"+tempFileName);
			cubeTableFile.setFileSize(file.getSize());
			
			//解析字段
			List<FileStructure> fileStructureList = JSON.parseArray(tableColumnList, FileStructure.class);
			//主键放第一个
			fileStructureList.add(0, new FileStructure(CubeConstants.COLUMN_PRIMARY_KEY, ViewColumnTypeEnum.BIGINT, "主键ID", true));
			//系统默认添加用户ID和主键
			fileStructureList.add(new FileStructure(CubeConstants.COLUMN_USER_ID, ViewColumnTypeEnum.BIGINT, "创建人ID", false));
			//2.根据数据源构造建表语句
			List<String> sqlList = DbCreateTableEnum.buildCreateTableSql(dsTypeEnum, tableName, tableComment, fileStructureList);
			log.info("SqlList: \n" + String.join("\n", sqlList));
			if(sqlList == null || sqlList.isEmpty()) {
				return R.fail("暂不支持的数据源");
			}
			//3.创建表
			dynamicTableService.executeSql(reportDatasource, sqlList.stream().toArray(String[]::new));
			//4.保存文件表信息
			cubeTableFileService.save(cubeTableFile);
			//5.批量插入数据
			//格式化表头<columnIndex, ViewColumnTypeEnum>
//			Map<Integer, ViewColumnTypeEnum> headerMap = fileStructureList.stream().collect(Collectors.toMap(FileStructure::getColumnIndex, FileStructure::getColumnType));
			//读取数据
//			String insertSql = CubeUtils.buildInsertSql(tableName, fileStructureList);
//			log.info("insertSql: " + insertSql);
			CubeExcelAnalysisEventListener analysisEventListener = new CubeExcelAnalysisEventListener(dataList -> {});
			EasyExcel.read(file.getInputStream(), analysisEventListener).sheet().doRead();
			dynamicTableService.batchSave(reportDatasource, tableName, SecureUtil.getUserId(), fileStructureList.stream().toArray(FileStructure[]::new), analysisEventListener.getDataList());
			return R.success("文件上传成功");
		} catch (Exception e) {
			log.error(e.getMessage(), e);
			return R.fail("文件上传失败");
		}
	}
	
	/**
	 * 
	 * @param file
	 * @param tableName
	 * @param dsId
	 * @return
	 * @throws IOException 
	@ApiOperation(value = "文件上传追加数据")
	@PostMapping("/uploadAppend")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "file", value = "文件", required = true),
		@ApiImplicitParam(name = "tableName", value = "表名", dataType = "string", required = true),
		@ApiImplicitParam(name = "dsId", value = "数据源ID", required = true),
	})
	public R<String> uploadAppend(MultipartFile file, String tableName, Integer dsId) throws IOException{
		// 1.查询数据源
		ReportDatasource reportDatasource = reportDatasourceService.getById(dsId);
		if (reportDatasource == null) {
			return R.fail("数据源不存在");
		}
		// 2.查询表结构
		List<ReportColumnDTO> reportColumnList = reportBaseInfoService.selectReportColumnList(reportDatasource,
				tableName);
		if(reportColumnList == null || reportColumnList.isEmpty()) {
			return R.fail("该表不存在");
		}
		List<FileStructure> fileStructureList = new ArrayList<FileStructure>();
		int index = 0;
		for(ReportColumnDTO columnDto : reportColumnList) {
			fileStructureList.add(new FileStructure(columnDto.getColumnName(), 
					 CubeConstants.COLUMN_USER_ID.equalsIgnoreCase(columnDto.getColumnName()) ? ViewColumnTypeEnum.BIGINT : columnDto.getColumnType(), 
					columnDto.getColumnComments(), index++));
		}
		Map<Integer, ViewColumnTypeEnum> headerMap = fileStructureList.stream().collect(Collectors.toMap(FileStructure::getColumnIndex, FileStructure::getColumnType));
		//读取数据
		String insertSql = CubeUtils.buildInsertSql(tableName, fileStructureList);
		log.info("insertSql: " + insertSql);
		try {
			CubeExcelAnalysisEventListener analysisEventListener = new CubeExcelAnalysisEventListener(dataList -> {});
			EasyExcel.read(file.getInputStream(), analysisEventListener).sheet().doRead();
			dynamicTableService.batchSave(reportDatasource, insertSql, SecureUtil.getUserId(), headerMap, analysisEventListener.getDataList());
			return R.success("追加数据成功");
		} catch (Exception e) {
			log.error("batchInsert Data Failed!!" + e.getMessage(), e);
			return R.fail("追加数据失败");
		}
//		try {
//			EasyExcel.read(file.getInputStream(), new CubeExcelAnalysisEventListener(dataList -> {
//				dynamicTableService.batchSave(reportDatasource, insertSql, SecureUtil.getUserId(), CubeUtils.getBatchSize(dsTypeEnum), headerMap, dataList);
//			})).sheet().doRead();
//			return R.success("追加数据成功");
//		} catch (Exception e) {
//			log.error("batchInsert Data Failed!!" + e.getMessage(), e);
//			return R.fail("追加数据失败");
//		}
	}
 */
}
