/**
 *
 */
package org.springblade.visual.service;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.Set;
import java.util.stream.Collectors;

import org.springblade.core.secure.BladeUser;
import org.springblade.core.tool.api.R;
import org.springblade.cube.enums.AggregationTypeEnum;
import org.springblade.cube.enums.ColumnAttrEnum;
import org.springblade.cube.enums.DateFormatEnum;
import org.springblade.cube.enums.DimensionTypeEnum;
import org.springblade.cube.enums.ViewColumnTypeEnum;
import org.springblade.cube.model.CubeData;
import org.springblade.cube.service.CubeDataPermissionService;
import org.springblade.cube.service.CubeDataService;
import org.springblade.cube.utils.CubeConstants;
import org.springblade.cube.utils.CubeUtils;
import org.springblade.report.entity.ReportDatasource;
import org.springblade.report.enums.DsTypeEnum;
import org.springblade.report.service.IReportDatasourceService;
import org.springblade.usergroup.dto.DefineGroupFilter;
import org.springblade.usergroup.dto.DefineRuleDto;
import org.springblade.visual.dto.ComponentColumn;
import org.springblade.visual.dto.ComponentFilter;
import org.springblade.visual.dto.ComponentFilterCondition;
import org.springblade.visual.dto.ComponentFilterDto;
import org.springblade.visual.dto.ComponentFilterItem;
import org.springblade.visual.dto.ComponentFilterPlus;
import org.springblade.visual.dto.ComponentFilterRelation;
import org.springblade.visual.dto.ComponentSort;
import org.springblade.visual.dto.CubePermParseDto;
import org.springblade.visual.dto.PreviewBaseDto;
import org.springblade.visual.dto.QuerySqlParam;
import org.springblade.visual.dto.WarningFilterRelativeTime;
import org.springblade.visual.dto.WarningFilterTime;
import org.springblade.visual.enums.CommonFunctionEnum;
import org.springblade.visual.enums.ComponentTypeEnum;
import org.springblade.visual.enums.FilterConnectTimeEnum;
import org.springblade.visual.enums.FilterConnectTypeEnum;
import org.springblade.visual.enums.FilterTimeCalcEnum;
import org.springblade.visual.utils.Tuple2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import lombok.extern.slf4j.Slf4j;

/**
 * @author wookvn
 *
 */
@Slf4j
@Service
public class QuerySqlService {

	@Autowired
	private CubeDataService cubeDataService;

	@Autowired
	private IReportDatasourceService reportDatasourceService;

	@Autowired
	private CubeDataPermissionService cubeDataPermissionService;

	/**
	 * 常规组件 - 解析查询SQL
	 * @param cubeDataId
	 * @param previewDto
	 * @return
	 */
	public R<QuerySqlParam> parseQuerySql(Integer cubeDataId, PreviewBaseDto previewDto, BladeUser bladeUser) {
		return parseQuerySql(cubeDataId, previewDto, bladeUser, ComponentTypeEnum.GENERAL);
	}

	/**
	 * 解析查询SQL
	 * @param cubeDataId
	 * @param previewDto
	 * @return
	 */
	public R<QuerySqlParam> parseQuerySql(Integer cubeDataId, PreviewBaseDto previewDto, BladeUser bladeUser, ComponentTypeEnum compType) {
		CubeData cubeData = cubeDataService.getById(cubeDataId);
		if(cubeData == null) {
			log.error("数据集不存在|id={}", cubeDataId);
			return R.fail("数据集不存在");
		}
		//2.查询数据源
		ReportDatasource reportDatasource = reportDatasourceService.getById(cubeData.getDatasourceId());
		if(reportDatasource == null) {
			log.error("数据源不存在|id={}", cubeData.getDatasourceId());
			return R.fail("数据源不存在");
		}
		DsTypeEnum dsTypeEnum = DsTypeEnum.resolve(reportDatasource.getDsType());
		//字段ID列表
		Set<Integer> columnIdList = new HashSet<Integer>();

		String fromSql = cubeData.getQuerySql();
		//2.构造Select模块
		List<ComponentColumn> columnList = JSON.parseArray(previewDto.getColumnList(), ComponentColumn.class);
		//初始化别名
		for(int i = 0; i < columnList.size(); i++) {
			ComponentColumn item = columnList.get(i);
			item.setColumnAliasName(CubeConstants.COLUMN_ALIAS_NAME_PREFIX + i);
			columnIdList.add(item.getCubeColumnId());
		}
		Map<Integer, ComponentColumn> columnMap = columnList.stream().collect(Collectors.toMap(ComponentColumn::getCubeColumnId, t -> t, (v1,v2) -> v1));
		List<String> selectSqlList = new ArrayList<String>();
		List<String> groupBySqlList = new ArrayList<String>();
		//主要用于排序时，不能使用带有%的字符串
		Map<Integer, String> originalNumberColumnMap = new HashMap<Integer, String>();

		for(ComponentColumn column : columnList) {
			String columnName = "";
			if(compType == ComponentTypeEnum.DETAIL && StrUtil.containsAnyIgnoreCase(column.getColumnSql(), CubeConstants.AGGREGATION_FUNCTIONS)) {
				return R.fail("清单组件不能包含聚合函数，请检查数据集字段【"+column.getColumnTitle()+"】");
			}
			//如果清单组件并且包含聚合函数，字段不参与到select中
			String selectColumnName = columnName = column.getColumnSql();
			//聚合函数[且计算字段是不包含聚合函数的]
			if(column.getAggregationType() != null && column.getDimensionType() == DimensionTypeEnum.ZHIBIAO
					&& compType != ComponentTypeEnum.DETAIL && Optional.<Integer>ofNullable(column.getVirtualAggregationStatus()).orElse(0) == 0) {
				selectColumnName = columnName = String.format(column.getAggregationType().getMethod(), selectColumnName);
			}
			// 格式化日期
			if (column.getTimeDataFormat() != null
					&& column.getTimeDataFormat() != DateFormatEnum.yyyy_MM_ddhh$mm$ss
					&& column.getViewDataType() == ViewColumnTypeEnum.DATETIME) {
				selectColumnName = columnName = CubeUtils.formatTimeColumn(column.getTimeDataFormat(), dsTypeEnum, selectColumnName);
			}
			// 格式化数字
			if (column.getNumberDataFormat() != null
					&& column.getViewDataType() == ViewColumnTypeEnum.NUMBER) {
				originalNumberColumnMap.put(column.getCubeColumnId(), selectColumnName);
				//groupby不需要百分号
				selectColumnName = columnName = String.format(column.getNumberDataFormat().getMethod(), selectColumnName);
				//select显示拼接百分号
				selectColumnName = CubeUtils.percentConcat(column.getNumberDataFormat(), dsTypeEnum, selectColumnName);
			}
			selectSqlList.add(selectColumnName + " as \"" + column.getColumnAliasName() + "\"");
			//如果是维度，添加到groupBy
			if(column.getDimensionType() == DimensionTypeEnum.WEIDU && compType != ComponentTypeEnum.DETAIL) {
				//ORACLE的groupBy不能使用别名，clickhouse和mysql可行
				ComponentColumn componentColumn = columnMap.get(column.getCubeColumnId());
				if(componentColumn != null && (dsTypeEnum == DsTypeEnum.clickhouse || dsTypeEnum == DsTypeEnum.mysql)) {
					groupBySqlList.add(componentColumn.getColumnAliasName());
				}else {
					groupBySqlList.add(columnName);
				}
			}
		}
		StringBuilder strBuilder = new StringBuilder("select ").append(String.join(",", selectSqlList))
				.append(" from ").append(fromSql);

		List<String> whereSqlList = new ArrayList<>();
		//解析权限组全局SQL
//		String permFilterSql = cubeDataPermissionService.parsePermFilterSql(cubeDataId, bladeUser);
		CubePermParseDto cubePermParseDto = cubeDataPermissionService.parseCubePermission(cubeDataId, bladeUser);
		if(StrUtil.isNotBlank(cubePermParseDto.getFilterSql())) {
			whereSqlList.add(cubePermParseDto.getFilterSql());
		}
		//构建过滤条件
		List<ComponentFilter> filterList = JSON.parseArray(previewDto.getFilterList(), ComponentFilter.class);
		filterList = Optional.ofNullable(filterList).orElse(new ArrayList<ComponentFilter>());
		//添加字段ID【主要用于：组件记录使用了哪些字段，便于血缘关系查询】
		filterList.forEach(f -> columnIdList.add(f.getColumn().getCubeColumnId()));
		//执行前置操作，是否有时间JSON，进行查询操作
		List<ComponentFilter> filterTimeList = parseFilterTime(previewDto.getFilterTimeJson(), dsTypeEnum);
		if(filterTimeList != null) {
			filterList.addAll(filterTimeList);
		}
		//解析过滤器以及过滤器关系
		ComponentFilterRelation componentFilterRelation = JSON.parseObject(previewDto.getFilterRelation(), ComponentFilterRelation.class);
		Tuple2<String, String> whereHavingSqlTuple = parseFilters(filterList, componentFilterRelation, dsTypeEnum);
		String whereSql = whereHavingSqlTuple.getT1(), havingSql = whereHavingSqlTuple.getT2();
		//where子句
		if(StrUtil.isNotBlank(whereSql)) {
			whereSqlList.add(whereSql);
		}
		if(!whereSqlList.isEmpty()) {
			strBuilder.append(" where " + String.join(" AND ", whereSqlList));
		}
		//清单组件不需要groupBy和having
		if(compType != ComponentTypeEnum.DETAIL) {
			//groupby子句【不能使用别名】
			if(!groupBySqlList.isEmpty()) {
				strBuilder.append(" group by ").append(String.join(",", groupBySqlList));
			}
			//having子句
			if(StrUtil.isNotBlank(havingSql)) {
				strBuilder.append(" having " + whereHavingSqlTuple.getT2());
			}
		}
		if(StrUtil.isNotEmpty(previewDto.getSortList())) {
			//构造排序条件
			List<ComponentSort> sortList =  JSON.parseArray(previewDto.getSortList(), ComponentSort.class);
			List<String> sortSqlList = new ArrayList<String>();
			for(ComponentSort sort : sortList) {
				ComponentColumn column = sort.getColumn();
				columnIdList.add(column.getCubeColumnId());
				//获取别名
				ComponentColumn componentColumn = columnMap.get(column.getCubeColumnId());
				if(componentColumn != null) { //如果别名不在select中，不能放到orderBy中
					//如果排序字段带有%，使用原始字段
					String orgialColumnName = originalNumberColumnMap.get(column.getCubeColumnId());
					String columnName = StrUtil.isNotBlank(orgialColumnName) ? orgialColumnName : componentColumn.getColumnAliasName();
					sortSqlList.add(columnName + " " + sort.getOrderType().name());
				}
			}
			if(!sortSqlList.isEmpty()) {
				strBuilder.append(" order by ").append(String.join(",", sortSqlList));
			}
		}
//		log.info("=====>>querySQL:" + strBuilder.toString());
		if(CubeUtils.checkSqlInject(strBuilder.toString())) {
			return R.fail("有SQL注入风险，请检查");
		}
		return R.data(new QuerySqlParam(strBuilder.toString(), cubeData, reportDatasource, columnList, columnIdList, cubePermParseDto.getExcludeColumnIdList()));
	}

	/**
	 * 常规组件 - 解析自定义人群查询SQL
	 * @return
	 */
	public R<QuerySqlParam> parseDefineGroupQuerySql(Integer cubeDataId, DefineRuleDto defineRuleDto, BladeUser bladeUser) {
		return parseDefineGroupQuerySql(cubeDataId, defineRuleDto, bladeUser, ComponentTypeEnum.GENERAL);
	}

	/**
	 * 解析自定义人群查询SQL
	 * @param cubeDataId
	 * @param defineRuleDto
	 * @param bladeUser
	 * @return
	 */
	public R<QuerySqlParam> parseDefineGroupQuerySql(Integer cubeDataId, DefineRuleDto defineRuleDto, BladeUser bladeUser, ComponentTypeEnum compType) {
		CubeData cubeData = cubeDataService.getById(cubeDataId);
		if(cubeData == null) {
			log.error("数据集不存在|id={}", cubeDataId);
			return R.fail("数据集不存在");
		}
		//2.查询数据源
		ReportDatasource reportDatasource = reportDatasourceService.getById(cubeData.getDatasourceId());
		if(reportDatasource == null) {
			log.error("数据源不存在|id={}", cubeData.getDatasourceId());
			return R.fail("数据源不存在");
		}
		DsTypeEnum dsTypeEnum = DsTypeEnum.resolve(reportDatasource.getDsType());
		//字段ID列表
		Set<Integer> columnIdList = new HashSet<Integer>();

		String fromSql = cubeData.getQuerySql();
		//处理电话号码字段
		List<ComponentColumn> phoneFieldList= JSON.parseArray(defineRuleDto.getPhoneField(), ComponentColumn.class);

		//构造Select模块
		List<ComponentColumn> columnList = JSON.parseArray(defineRuleDto.getColumns(), ComponentColumn.class);

		//将手机号字段放到最开始位置
		columnList.add(0,phoneFieldList.get(0));
		//初始化别名
		for(int i = 0; i < columnList.size(); i++) {
			ComponentColumn item = columnList.get(i);
			item.setColumnAliasName(CubeConstants.COLUMN_ALIAS_NAME_PREFIX + i);
			columnIdList.add(item.getCubeColumnId());
		}
		Map<Integer, ComponentColumn> columnMap = columnList.stream().collect(Collectors.toMap(ComponentColumn::getCubeColumnId, t -> t, (v1,v2) -> v1));
		List<String> selectSqlList = new ArrayList<String>();
		List<String> groupBySqlList = new ArrayList<String>();
		//主要用于排序时，不能使用带有%的字符串
		Map<Integer, String> originalNumberColumnMap = new HashMap<Integer, String>();
		//用于存储having的aggregationType
		Map<Integer, AggregationTypeEnum> havingMap = new HashMap<Integer, AggregationTypeEnum>();
		for(ComponentColumn column : columnList) {
			String columnName = "";
			if(compType == ComponentTypeEnum.DETAIL && StrUtil.containsAnyIgnoreCase(column.getColumnSql(), CubeConstants.AGGREGATION_FUNCTIONS)) {
				return R.fail("清单组件不能包含聚合函数，请检查数据集字段【"+column.getColumnTitle()+"】");
			}
			//如果清单组件并且包含聚合函数，字段不参与到select中
			String selectColumnName = columnName = column.getColumnSql();
			//聚合函数
			if(column.getColumnAttr()!= ColumnAttrEnum.VIRTUAL && column.getAggregationType() != null && column.getDimensionType() == DimensionTypeEnum.ZHIBIAO && compType != ComponentTypeEnum.DETAIL) {
				selectColumnName = columnName = String.format(column.getAggregationType().getMethod(), selectColumnName);
				havingMap.put(column.getCubeColumnId(),column.getAggregationType());
			}
			// 格式化日期
			if (column.getTimeDataFormat() != null
					&& column.getTimeDataFormat() != DateFormatEnum.yyyy_MM_ddhh$mm$ss
					&& column.getViewDataType() == ViewColumnTypeEnum.DATETIME) {
				selectColumnName = columnName = CubeUtils.formatTimeColumn(column.getTimeDataFormat(), dsTypeEnum, selectColumnName);
			}
			// 格式化数字
			if (column.getNumberDataFormat() != null
					&& column.getViewDataType() == ViewColumnTypeEnum.NUMBER) {
				originalNumberColumnMap.put(column.getCubeColumnId(), selectColumnName);
				//groupby不需要百分号
				selectColumnName = columnName = String.format(column.getNumberDataFormat().getMethod(), selectColumnName);
				//select显示拼接百分号
				selectColumnName = CubeUtils.percentConcat(column.getNumberDataFormat(), dsTypeEnum, selectColumnName);
			}
			selectSqlList.add(selectColumnName + " as \"" + column.getColumnAliasName() + "\"");
			//如果是维度，添加到groupBy
			if(column.getDimensionType() == DimensionTypeEnum.WEIDU && compType != ComponentTypeEnum.DETAIL) {
				//ORACLE的groupBy不能使用别名，clickhouse和mysql可行
				ComponentColumn componentColumn = columnMap.get(column.getCubeColumnId());
				if(componentColumn != null && (dsTypeEnum == DsTypeEnum.clickhouse || dsTypeEnum == DsTypeEnum.mysql)) {
					groupBySqlList.add(componentColumn.getColumnAliasName());
				}else {
					groupBySqlList.add(columnName);
				}
			}
		}
		StringBuilder strBuilder = new StringBuilder("select ").append(String.join(",", selectSqlList))
				.append(" from ").append(fromSql);

		List<String> whereSqlList = new ArrayList<>();
		//解析权限组全局SQL
		CubePermParseDto cubePermParseDto = cubeDataPermissionService.parseCubePermission(cubeDataId, bladeUser);
		if(StrUtil.isNotBlank(cubePermParseDto.getFilterSql())) {
			whereSqlList.add(cubePermParseDto.getFilterSql());
		}
		//构建where条件
		String whereSql="";
		if (StringUtils.isNotBlank(defineRuleDto.getFilter())){
			DefineGroupFilter filter = JSONObject.parseObject(defineRuleDto.getFilter(), DefineGroupFilter.class);
			List<ComponentFilter> filterList = JSON.parseArray(filter.getConditions(),ComponentFilter.class);
			filterList = Optional.ofNullable(filterList).orElse(new ArrayList<ComponentFilter>());
			//排除掉虚拟字段并且带聚合函数
			List<ComponentFilter> componentFilters =new ArrayList<>();
			for(ComponentFilter columnFilter : filterList) {
				ComponentColumn column = columnFilter.getColumn();
				if (column.getColumnAttr()!= ColumnAttrEnum.VIRTUAL && column.getAggregationType() != null){
					if (havingMap.containsKey(column.getCubeColumnId())){
						AggregationTypeEnum method = havingMap.get(column.getCubeColumnId());
						column.setAggregationType(method);
					}
					componentFilters.add(columnFilter);
				}
			}
			Tuple2<String, String> whereTuple = parseDefineFilters(componentFilters, filter.getRelation(),filter.getFilters(), dsTypeEnum);
			whereSql = whereTuple.getT1();
		}
		//构建having条件
		String havingSql="";
		if (StringUtils.isNotBlank(defineRuleDto.getHaving())){
			DefineGroupFilter having = JSONObject.parseObject(defineRuleDto.getHaving(), DefineGroupFilter.class);
			List<ComponentFilter> havingList = JSON.parseArray(having.getConditions(),ComponentFilter.class);
			havingList = Optional.ofNullable(havingList).orElse(new ArrayList<ComponentFilter>());
			Tuple2<String, String> havingTuple = parseDefineFilters(havingList, having.getRelation(),having.getFilters(), dsTypeEnum);
			havingSql = havingTuple.getT2();
		}
		//where子句
		if(StringUtils.isNotBlank(whereSql)) {
			whereSqlList.add(whereSql);
		}
		if(!whereSqlList.isEmpty()) {
			strBuilder.append(" where " + String.join(" AND ", whereSqlList));
		}
//		清单组件不需要groupBy和having
		if(compType != ComponentTypeEnum.DETAIL) {
			//groupby子句【不能使用别名】
			if(!groupBySqlList.isEmpty()) {
				strBuilder.append(" group by ").append(String.join(",", groupBySqlList));
			}
			//having子句
			if(StringUtils.isNotBlank(havingSql)) {
				strBuilder.append(" having " + havingSql);
			}
		}
//		log.info("=====>>querySQL:" + strBuilder.toString());
		if(CubeUtils.checkSqlInject(strBuilder.toString())) {
			return R.fail("有SQL注入风险，请检查");
		}
		return R.data(new QuerySqlParam(strBuilder.toString(), cubeData, reportDatasource, columnList, columnIdList, cubePermParseDto.getExcludeColumnIdList()));
	}



	/**
	 * 解析自定义人群过滤器
	 * @return
	 */
	public Tuple2<String, String> parseDefineFilters(List<ComponentFilter> filterList, FilterConnectTypeEnum relation,String filters, DsTypeEnum dsTypeEnum){
		String whereSql = "", havingSql = "";
		if(filterList != null && !filterList.isEmpty()) {

			//过滤关系
			if(relation != null) {
				List<String> havingSqlList = new ArrayList<String>();
				//1.构造单个条件集合
				List<String> whereSqlList = new ArrayList<String>();

				Tuple2<String, String> tuple = _buildConditions(filterList, relation, dsTypeEnum);
				if(StringUtils.isNotBlank(tuple.getT1())) {
					whereSqlList.add(tuple.getT1());
				}
				if(StringUtils.isNotBlank(tuple.getT2())){
					havingSqlList.add(tuple.getT2());
				}
				//2.构造多个条件集合
				List<DefineGroupFilter> componentFilterList =JSON.parseArray(filters,DefineGroupFilter.class);
				if (componentFilterList != null && componentFilterList.size()>0){
					for (DefineGroupFilter filter : componentFilterList){
						List<ComponentFilter> componentFilters = JSON.parseArray(filter.getConditions(),ComponentFilter.class);
						FilterConnectTypeEnum connectTypeEnum = filter.getRelation();
						if (componentFilters != null && componentFilters.size()>0  && connectTypeEnum!= null){
							Tuple2<String, String> tuple2 = _buildConditions(componentFilters, connectTypeEnum, dsTypeEnum);
							if(StringUtils.isNotBlank(tuple2.getT1())) {
								whereSqlList.add(tuple2.getT1());
							}
							if(StrUtil.isNotBlank(tuple.getT2())){
								havingSqlList.add(tuple.getT2());
							}
						}
					}
				}
				whereSql =_stringListJoin(whereSqlList, relation);
				havingSql=_stringListJoin(havingSqlList, relation);
			}else { //没有过滤关系
				Tuple2<String, String> whereHavingSqlTuple = _buildConditions(filterList, FilterConnectTypeEnum.AND, dsTypeEnum);
				if(whereHavingSqlTuple != null) {
					whereSql = whereHavingSqlTuple.getT1();
					havingSql = whereHavingSqlTuple.getT2();
				}
			}
		}
		return new Tuple2<String, String>(whereSql, havingSql);
	}



	/**
	 * 解析过滤器
	 * @param filterList
	 * @param componentFilterRelation
	 * @return
	 */
	public Tuple2<String, String> parseFilters(List<ComponentFilter> filterList, ComponentFilterRelation componentFilterRelation, DsTypeEnum dsTypeEnum){
		String whereSql = "", havingSql = "";
		if(filterList != null && !filterList.isEmpty()) {
			//过滤关系
			if(componentFilterRelation != null) {
				//filterRelation包含where和having的过滤关系，依次处理
				Tuple2<String, String> whereSqlTuple = _buildFilters(componentFilterRelation.getWhere(), filterList, dsTypeEnum);
				if(whereSqlTuple != null) {
					whereSql = whereSqlTuple.getT1();
				}
				Tuple2<String, String> havingSqlTuple = _buildFilters(componentFilterRelation.getHaving(), filterList, dsTypeEnum);
				if(havingSqlTuple != null) {
					havingSql = havingSqlTuple.getT2();
				}
				//构造非uniqId的条件集合【主要是全局筛选器，因此只需要处理where】
				List<ComponentFilter> dataList = filterList.stream().filter(f -> StrUtil.isEmpty(f.getUniqId())).collect(Collectors.toList());
				if(!dataList.isEmpty()) {
					Tuple2<String, String> globalWhereSqlTuple = _buildConditions(dataList, FilterConnectTypeEnum.AND, dsTypeEnum);
					if(globalWhereSqlTuple != null && !"".equals(globalWhereSqlTuple.getT1())) {
						whereSql += (StrUtil.isNotBlank(whereSql) ? " AND " : "") + globalWhereSqlTuple.getT1();
					}
				}
			}else { //没有过滤关系
				Tuple2<String, String> whereHavingSqlTuple = _buildConditions(filterList, FilterConnectTypeEnum.AND, dsTypeEnum);
				if(whereHavingSqlTuple != null) {
					whereSql = whereHavingSqlTuple.getT1();
					havingSql = whereHavingSqlTuple.getT2();
				}
			}
		}
		return new Tuple2<String, String>(whereSql, havingSql);
	}

	/**
	 * 解析相对时间和标准时间
	 * @param filterTimeJson
	 * @return
	 */
	public List<ComponentFilter> parseFilterTime(String filterTimeJson, DsTypeEnum dsTypeEnum){
		if(StrUtil.isEmpty(filterTimeJson)) {
			return null;
		}
		WarningFilterTime filterTime = JSON.parseObject(filterTimeJson, WarningFilterTime.class);
		//时间字段和相对时间对象 不能为空
		if(filterTime.getTimeColumn() == null || filterTime.getRelativeTime() == null) {
			return null;
		}
		if(filterTime.getCalcType() == FilterTimeCalcEnum.SPECIAL &&
				filterTime.getTimeColumn().getViewDataType() != ViewColumnTypeEnum.NUMBER) {
			log.warn("timeColumn is not a Number column!!");
			return null;
		}
		if(filterTime.getCalcType() == FilterTimeCalcEnum.NORMAL &&
				filterTime.getTimeColumn().getViewDataType() != ViewColumnTypeEnum.DATETIME) {
			log.warn("timeColumn is not a DateTime column!!");
			return null;
		}
		//计算出2个时间[开始时间和结束时间]
		WarningFilterRelativeTime relativeTime = filterTime.getRelativeTime();
		LocalDateTime startDateTime = relativeTime.getLeftModifier().getTime(relativeTime.getLeftVal(), filterTime.getTimeType());
		LocalDateTime endDateTime = relativeTime.getRightModifier().getTime(relativeTime.getRightVal(), filterTime.getTimeType());
		List<Object> values = new ArrayList<Object>();
		if(filterTime.getCalcType() == FilterTimeCalcEnum.NORMAL) {
			ComponentColumn timeColumn = filterTime.getTimeColumn();
			values.add(DateUtil.format(startDateTime, timeColumn.getTimeDataFormat().getFormat()));
			values.add(DateUtil.format(endDateTime, timeColumn.getTimeDataFormat().getFormat()));
		}else {
			values.add(filterTime.getTimeType().formatInteger(startDateTime));
			values.add(filterTime.getTimeType().formatInteger(endDateTime));
		}
		List<ComponentFilter> filterList = new ArrayList<ComponentFilter>();
		//构造时间字段过滤器
		ComponentFilter timeColumnFilter = new ComponentFilter(filterTime.getTimeColumn());
		timeColumnFilter.setConditions(Arrays.asList(new ComponentFilterCondition(CommonFunctionEnum.between, values)));
		filterList.add(timeColumnFilter);
		//如果时间计算方式为特殊时间，需要添加 时间纬度过滤器
		if(filterTime.getCalcType() == FilterTimeCalcEnum.SPECIAL && filterTime.getTimeTypeColumn() != null) {
			ComponentFilter timeTypeColumnFilter = new ComponentFilter(filterTime.getTimeTypeColumn());
			timeTypeColumnFilter.setConditions(Arrays.asList(new ComponentFilterCondition(CommonFunctionEnum.eq, Arrays.asList(filterTime.getTimeType().name()))));
			filterList.add(timeTypeColumnFilter);
		}
		return filterList;
	}

	/**
	 * 构建Where子句和Having子句
	 * @param compFilterDto
	 */
	private Tuple2<String, String> _buildFilters(ComponentFilterDto compFilterDto, List<ComponentFilter> filterList, DsTypeEnum dsTypeEnum) {
		if((compFilterDto == null
				|| compFilterDto.getConditions() == null && compFilterDto.getFilters() == null)
				|| (compFilterDto.getConditions().isEmpty() && compFilterDto.getFilters().isEmpty())
				|| compFilterDto.getRelation() == null) {
			return null;
		}
		List<String> havingSqlList = new ArrayList<String>();
		//1.构造单个条件集合
		List<String> whereSqlList = new ArrayList<String>();
		if(compFilterDto.getConditions() != null && !compFilterDto.getConditions().isEmpty()) {
			List<String> uniqIdList = compFilterDto.getConditions().stream().filter(f -> StrUtil.isNotBlank(f.getUniqId())).map(ComponentFilterItem::getUniqId).collect(Collectors.toList());
			List<ComponentFilter> filterDataList = filterList.stream().filter(f -> uniqIdList.contains(f.getUniqId())).collect(Collectors.toList());
			Tuple2<String, String> tuple = _buildConditions(filterDataList, compFilterDto.getRelation(), dsTypeEnum);
			if(StrUtil.isNotBlank(tuple.getT1())) {
				whereSqlList.add(tuple.getT1());
			}
			if(StrUtil.isNotBlank(tuple.getT2())){
				havingSqlList.add(tuple.getT2());
			}
		}
		//2.构造多个条件集合
		if(compFilterDto.getFilters() != null && !compFilterDto.getFilters().isEmpty()) {
			for(ComponentFilterPlus filterPlus : compFilterDto.getFilters()) {
				List<String> uniqIdList = filterPlus.getConditions().stream().filter(f -> StrUtil.isNotBlank(f.getUniqId())).map(ComponentFilterItem::getUniqId).collect(Collectors.toList());
				List<ComponentFilter> conds = filterList.stream().filter(f -> uniqIdList.contains(f.getUniqId())).collect(Collectors.toList());
				FilterConnectTypeEnum relation = filterPlus.getRelation();
				if(conds != null && relation != null && !conds.isEmpty()) {
					Tuple2<String, String> tuple = _buildConditions(conds, relation, dsTypeEnum);
					if(StrUtil.isNotBlank(tuple.getT1())) {
						whereSqlList.add(tuple.getT1());
					}
					if(StrUtil.isNotBlank(tuple.getT2())){
						havingSqlList.add(tuple.getT2());
					}
				}
			}
		}
		return new Tuple2<String, String>(_stringListJoin(whereSqlList, compFilterDto.getRelation()), _stringListJoin(havingSqlList, compFilterDto.getRelation()));
	}

	/**
	 * 构造查询条件
	 * @param compFilterList
	 * @param relation
	 * @param dsTypeEnum
	 * @return
	 */
	private Tuple2<String, String> _buildConditions(List<ComponentFilter> compFilterList, FilterConnectTypeEnum relation, DsTypeEnum dsTypeEnum) {
		List<String> whereSqlList = new ArrayList<String>();
		List<String> havingSqlList = new ArrayList<String>();
		for(ComponentFilter columnFilter : compFilterList) {
			ComponentColumn column = columnFilter.getColumn();
			String columnName = column.getColumnSql();
			//2.计算过滤时机
			FilterConnectTimeEnum connectTime = columnFilter.getConnectTime();
			//HAVING需要把聚合函数参与进来[如果计算字段已存在聚合函数，不再添加]
			if(connectTime == FilterConnectTimeEnum.HAVING && column.getAggregationType() != null
					&& Optional.<Integer>ofNullable(column.getVirtualAggregationStatus()).orElse(0) == 0) {
				columnName = String.format(column.getAggregationType().getMethod(), columnName);
			}
			//格式化日期
			if (column.getTimeDataFormat() != null
//					&& column.getTimeDataFormat() != DateFormatEnum.yyyy_MM_ddhh$mm$ss
					&& column.getViewDataType() == ViewColumnTypeEnum.DATETIME) {
				columnName = CubeUtils.formatTimeColumn(column.getTimeDataFormat(), dsTypeEnum, columnName);
			}
			//格式化数字
//			if (column.getNumberDataFormat() != null
//					&& column.getViewDataType() == ViewColumnTypeEnum.NUMBER) {
//				columnName = String.format(column.getNumberDataFormat().getMethod(), columnName);
//			}
			//判断columnName是否为空
			if(StrUtil.isEmpty(columnName)) {
				continue;
			}
			//3.计算过滤条件
			List<ComponentFilterCondition> conditions = columnFilter.getConditions();
			if(conditions != null && !conditions.isEmpty()) {
				List<String> conditionSqlList = new ArrayList<String>();
				//4.添加过滤条件
				for(ComponentFilterCondition condition : conditions) {
					String conditionSql = condition.getFunction().handle(columnName, column.getViewDataType(), condition.getValues(), dsTypeEnum);
					if(!StrUtil.isEmpty(conditionSql)) {
						conditionSqlList.add(conditionSql);
					}
				}
				if(!conditionSqlList.isEmpty()) {
					//不同的过滤时机添加到对应的集合
					if(connectTime == FilterConnectTimeEnum.HAVING) {
						havingSqlList.add(_stringListJoin(conditionSqlList, columnFilter.getConnectType()));
					}else {
						whereSqlList.add(_stringListJoin(conditionSqlList, columnFilter.getConnectType()));
					}
				}
			}
		}
		return new Tuple2<String, String>(_stringListJoin(whereSqlList, relation), _stringListJoin(havingSqlList, relation));
	}

	private String _stringListJoin(List<String> dataList, FilterConnectTypeEnum relation) {
		if(dataList == null || dataList.isEmpty()) {
			return "";
		}
		String startParenthesis = dataList.size() > 1 ? "(" : "";
		String endParenthesis = dataList.size() > 1 ? ")" : "";
		return startParenthesis + String.join(" " + relation.name() + " ", dataList) + endParenthesis;
	}
}
